<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * MDB admin container for maintaining Auth/MDB
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::MDB class.
 */
require_once 'LiveUser/Admin/Auth/Common.php';
require_once 'MDB.php';

/**
 * Class LiveUser_Admin_Auth_MDB
 *
 * Simple MDB-based complexity driver for LiveUser.
 *
 * Description:
 * This admin class provides the following functionalities
 * - adding users
 * - removing users
 * - update user data (auth related: username, pwd, active)
 * - adding rights
 * - removing rights
 * - get all users
 *
 * ATTENTION:
 * This class is only experimental. API may change. Use it at your own risk.
 *
 * @author  Bjrn Kraus <krausbn@php.net>
 * @version $Id: MDB.php,v 1.11 2003/08/21 08:05:24 lsmith Exp $
 * @package LiveUser
 */
class LiveUser_Admin_Auth_Container_MDB extends LiveUser_Admin_Auth_Common
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused).
     *
     * @var    string
     * @access private
     */
    var $dsn = null;

    /**
     * PEAR::MDB connection object.
     *
     * @var    object
     * @access private
     */
    var $dbc = null;

    /**
     * Auth table
     * Table where the auth data is stored.
     * @var string
     * @access public
     */
    var $authTable = 'liveuser_users';

    /**
     * Columns of the auth table.
     * Associative array with the names of the auth table columns.
     * The 'userId', 'handle' and 'password' fields have to be set.
     * 'lastLogin' and 'isActive' are optional.
     * It doesn't make sense to set only one of the time columns without the
     * other.
     *
     * @var array
     * @access public
     */
    var $authTableCols = array('userId'    => 'user_id',
                               'handle'    => 'handle',
                               'password'  => 'password',
                               'lastLogin' => 'lastlogin',
                               'isActive'  => 'is_active');

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var    boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * Constructor
     *
     * The second parameters expects an array containing the parameters
     * for the given container.
     * 
     * Say you have a conf array like that
     * <code>
     * 'authContainers' => array(
     *     array(
     *         'type'          => 'MDB',
     *         'loginTimeout'  => 0,
     *         'expireTime'    => 3600,
     *         'idleTime'      => 1800,
     *         'allowDuplicateHandles' => 0,
     *         'authTable'     => 'users',
     *         'authTableCols' => array(
     *             'userId'       => 'user_id',
     *             'handle'       => 'handle',
     *             'password'     => 'password',
     *             'lastLogin'    => 'lastlogin',
     *             'isActive'     => 'is_active'
     *         )
     *     )
     * ),
     * </code>
     * 
     * This class expectsonly the array concerning the authcontainer you which
     * to do the administration on. This way if you have several DB based
     * auth container you can specify which one you are acting on.
     * 
     * <code>
     * $dsn = array('dsn' => "type://user:pass@host/database");
     * 
     * $conf = 
     *     array(
     *         'type'          => 'MDB',
     *         'loginTimeout'  => 0,
     *         'expireTime'    => 3600,
     *         'idleTime'      => 1800,
     *         'allowDuplicateHandles' => 0,
     *         'authTable'     => 'users',
     *         'authTableCols' => array(
     *             'userId'       => 'user_id',
     *             'handle'       => 'handle',
     *             'password'     => 'password',
     *             'lastLogin'    => 'lastlogin',
     *             'isActive'     => 'is_active'
     *         )
     *     );
     * 
     * $obj = new LiveUser_Admin_Auth_Container_DB($dsn, $conf);
     * </code>
     * 
     * @access protected
     * @param  mixed  Array or PEAR::MDB object.
     * @param  array  configuration array
     * @return void
     */
    function &LiveUser_Admin_Auth_Container_MDB(&$connectOptions, $conf)
    {
        if (is_object($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }

            $this->dbc = MDB::connect($connectOptions['dsn']);

            if (!MDB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }

        if ($conf['type'] != 'MDB') {
            return PEAR::raiseError('the configuration array does not match
            this container type. Please use an array for a MDB container.');
        }

        $this->authTableCols = array_merge($conf['authTableCols'], $this->authTableCols);
        $this->authTable     = $conf['authTable'];
    } // end func LiveUser_Admin_Auth_MDB

    /**
     * Adds a new user to Auth/MDB.
     *
     * @access  public
     * @param   string  Handle (username).
     * @param   string  Password.
     * @param   boolean Sets the user active (1) or not (0).
     * @return  mixed   Users auth ID on success, DB error if not.
     */
    function addUser($handle, $password = '', $active = true)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Generate new user ID
        $authId = md5($handle.getmypid().uniqid(rand(),1));

        // Register new user in auth table
        $query = '
            INSERT INTO
                ' . $this->authTable . '

                (
                ' . $this->authTableCols['userId'] . ',
                ' . $this->authTableCols['handle'] . ',
                ' . $this->authTableCols['password'] . ',
                ' . $this->authTableCols['isActive'] . '
                )

            VALUES
                (
                ' . $this->dbc->getTextValue($authId) . ',
                ' . $this->dbc->getTextValue($handle) . ',
                ' . $this->dbc->getTextValue($this->encryptPW($password)) . ',
                ' . $this->dbc->getBooleanValue($active) . '
                )';

        $result = $this->dbc->query($query);

        if (MDB::isError($result)) {
            return $result;
        }

        return $authId;
    } // end func addUser

    /**
     * Removes an existing user from Auth/MDB.
     *
     * @access  public
     * @param   string   $authId   Auth user ID of the user that should be removed.
     * @return  mixed              True on success, MDB error if not.
     */
    function removeUser($authId)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Delete user from auth table (MDB/Auth)
        $query = '
            DELETE FROM
                ' . $this->authTable . '
            WHERE
                user_id = ' . $this->dbc->getTextValue($authId);

        $result = $this->dbc->query($query);

        if (MDB::isError($result)) {
            return $result;
        }

        return true;
    } // end func removeUser

    /**
     * Changes user data in auth table.
     *
     * @access  public
     * @access  string   Auth user ID.
     * @param   string   Handle (username) (optional).
     * @param   string   Password (optional).
     * @param   boolean  Sets the user active (1) or not (0) (optional).
     * @return  mixed    True on success, DB error if not.
     */
    function updateUser($authId, $handle = '', $password = '', $active = null)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Create query.
        $query = '
            UPDATE
                ' . $this->authTable . '
            SET ';

        if (!empty($handle)) {
            $updateValues[] =
                $this->authTableCols['handle'] . '=' . $this->dbc->getTextValue($handle);
        }
        if (!empty($password)) {
            $updateValues[] =
                $this->authTableCols['password'] . ' = ' . $this->dbc->getTextValue($this->encryptPW($password));
        }
        if (isset($active)) {
            $updateValues[] =
                $this->authTableCols['isActive'] . ' = ' . $this->dbc->getBooleanValue($active);
        }

        if (count($updateValues) >= 1) {
            $query .= implode(', ', $updateValues);
        } else {
            return false;
        }

        $query .= 'WHERE
            ' . $this->authTableCols['userId'] . '=' . $this->dbc->getTextValue($authId);

        $result = $this->dbc->query($query);

        if (MDB::isError($result)) {
            return $result;
        }

        return true;
    } // end func changeUser

    /**
     * Gets all users with handle, password, authId, permId, currentLogin,
     * lastLogin, isActive and individual rights.
     *
     * The array will look like this:
     * $userData[0]['authId']       = 'wujha433gawefawfwfiuj2ou9823r98h';
     *             ['handle']       = 'myLogin';
     *             ['password']     = 'd346gs2gwaeiuhaeiuuweijfjuwaefhj';
     *             ['lastLogin']    = 1254801292; (Unix timestamp)
     *             ['isActive']     = 1; (1 = yes, 0 = no)
     *
     * @access  public
     * @return  mixed                 Array with user data or MDB error.
     */
    function getAllUsers()
    {
        if (!$this->init_ok) {
            return false;
        }

        // First: Get all data from auth table.
        $query = '
            SELECT
                ' . $this->authTableCols['userId']    . ' AS authId,
                ' . $this->authTableCols['handle']    . ' AS handle,
                ' . $this->authTableCols['password']  . ' AS password,
                ' . $this->authTableCols['lastLogin'] . ' AS lastLogin,
                ' . $this->authTableCols['isActive']  . ' AS isActive
            FROM
                ' . $this->authTable;

        $types = array('text', 'text', 'text', 'timestamp', 'boolean');

        $userData = $this->dbc->queryAll($query, $types, MDB_FETCHMODE_ASSOC, true);

        if (MDB::isError($userData)) {
            return $userData;
        }

        return $userData;
    } // end func getAllUsers
}
?>