<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * DB_Complex permission administration class
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require the parent class definition
 */
require_once 'LiveUser/Admin/Perm/DB_Common.php';
require_once 'LiveUser/Perm/Common.php';

/**
 * This is a PEAR::DB admin class for the LiveUser package.
 *
 * It takes care of managing the permission part of the package.
 *
 * A PEAR::DB connection object can be passed to the constructor to reuse an
 * existing connection. Alternatively, a DSN can be passed to open a new one.
 *
 * Requirements:
 * - Files "common.php", "Container/DB_Complex.php" in directory "Perm"
 * - Array of connection options must be passed to the constructor.
 *   Example: array("server" => "localhost", "user" => "root",
 *   "password" => "pwd", "database" => "AllMyPreciousData")
 *
 * @author  Christian Dickmann <dickmann@php.net>
 * @author  Markus Wolff <wolff@21st.de>
 * @author  Matt Scifo <mscifo@php.net>
 * @version $Id: DB_Complex.php,v 1.14 2003/08/18 18:30:38 arnaud Exp $
 * @package LiveUser
 */
class LiveUser_Admin_Perm_Container_DB_Complex extends LiveUser_Admin_Perm_DB_Common
{
    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var    boolean
     * @access private
     */
    var $init_ok = false;

    /**
     * Constructor
     *
     * The second parameters expects the smae array as you use
     * when instantiating LiveUser::factory()
     *
     * @access protected
     * @param  mixed  Array or PEAR::DB object.
     * @param  array  configuration array
     * @return void
     * @see    LiveUser::factory()
     */
    function &LiveUser_Admin_Container_Perm_DB_Complex(&$connectOptions, $conf)
    {
        if (is_object($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }

            $this->dbc = DB::connect($connectOptions['dsn']);

            if (!DB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }

        if (isset($conf['permContainer']['prefix'])) {
            $this->prefix = $conf['permContainer']['prefix'];
        }
    }

    /**
     * Add a group to the database
     *
     * @access public
     * @param  string  name of group constant
     * @param  string  name of group
     * @param  boolean description of group
     * @param  boolean activate group?
     * @param  integer owner_user_id of group
     * @param  integer owner_group_id of group
     * @return mixed integer (group_id) or DB Error object
     */
    function addGroup($define_name, $group_name, $group_comment = null, $active = false, $owner_user = null, $owner_group = null)
    {
        // Get next group ID
        $group_id = $this->dbc->nextId($this->prefix . 'groups');
        if (DB::isError($group_id)) {
            return $group_id;
        };

        // Insert Group into Groupstable
        $query = 'INSERT INTO
                  ' . $this->prefix . 'groups
                  (group_id, group_define_name, owner_user_id, owner_group_id, is_active)
                VALUES
                  (
                    ' . (int)$group_id . ',
                    ' . $this->dbc->quote($define_name) . ',
                    ' . (($owner_user  === null) ? 'null' : $this->dbc->quote($owner_user)) . ',
                    ' . (($owner_group === null) ? 'null' : (int)$owner_group) . ',
                    "' . ($active ? 'Y' : 'N') . '"
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Insert Group tranlation into Tranlastions table
        $result = $this->addTranslation(
            $group_id,
            LIVEUSER_SECTION_GROUP,
            $this->getCurrentLanguage(),
            $group_name,
            $group_comment
        );
        if (DB::isError($result)) {
            return $result;
        };

        return $group_id;
    }

    /**
     * assign subgroup to parent group
     *
     * @access public
     * @param  integer id of parent group
     * @param  integer id of child group
     * @return mixed boolean or DB Error object
     */
    function assignSubgroup($group_id, $subgroup_id)
    {
        // XXX test for infinite loop
        $query = 'INSERT INTO
                  ' . $this->prefix . 'group_subgroups
                  (group_id, subgroup_id)
                VALUES
                  (
                    ' . (int)$group_id . ',
                    ' . (int)$subgroup_id . '
                  )';
        $result = $this->dbc->query($query);

        return $result;
    }

    /**
     * Deletes a group from the database
     *
     *
     * @access public
     * @param  integer id of deleted group
     * @param  boolean recursive delete of all subgroups
     * @return mixed   boolean or DB Error object
     */
    function removeGroup($group_id, $recursive = false)
    {
        // Recursive delete groups
        if ($recursive) {
            // Get all subgroups
            $query = 'SELECT
                      subgroup_id
                    FROM
                      ' . $this->prefix . 'group_subgroups
                    WHERE
                      group_id = ' . (int)$group_id;

            $result = $this->dbc->getCol($query);

            if (DB::isError($result)) {
                return $result;
            };

            // Recursive removeGroup() call for every subgroup
            foreach ($result as $subgroup_id) {
                $res = $this->removeGroup($subgroup_id, true);
                if (DB::isError($res)) {
                    return $res;
                };
            };
        };

        // Delete group itself
        $query = 'DELETE FROM
                  ' . $this->prefix . 'groups
                WHERE
                  group_id = ' . (int)$group_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete Subgroup assignments
        $query = 'DELETE FROM
                  ' . $this->prefix . 'group_subgroups
                WHERE
                  group_id = ' . (int)$group_id . ' OR
                  subgroup_id = ' . (int)$group_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete group translations
        $result = $this->removeTranslation($group_id, LIVEUSER_SECTION_GROUP, $this->getCurrentLanguage(), true);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete user assignments
        $query = 'DELETE FROM
                  ' . $this->prefix . 'groupusers
                WHERE
                  group_id = ' . (int)$group_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete group rights
        $query = 'DELETE FROM
                  ' . $this->prefix . 'grouprights
                WHERE
                  group_id = ' . (int)$group_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Update group
     *
     * @access public
     * @param  string  id of group
     * @param  string  name of group
     * @param  boolean description of group
     * @param  boolean activate group?
     * @param  integer owner_user_id of group
     * @param  integer owner_group_id of group
     * @return mixed   boolean or DB Error object
     */
    function updateGroup($group_id, $group_name, $group_comment = null, $active = false, $owner_user = null, $owner_group = null)
    {
        $query = 'UPDATE
                  ' . $this->prefix . 'groups
                SET
                  owner_user_id  = ' . (($owner_user  !== null) ? $this->dbc->quote($owner_user) : 'null') . ',
                  owner_group_id = ' . (($owner_group !== null) ? (int)$owner_group : 'null') . ',
                  is_active      = ' . ($active ? '"Y"' : '"N"') . '
                WHERE
                  group_id = ' . (int)$group_id;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Update Group tranlation into Tranlastions table
        $result = $this->updateTranslation(
            $group_id,
            LIVEUSER_SECTION_GROUP,
            $this->getCurrentLanguage(),
            $group_name,
            $group_comment
        );
        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Activate group
     *
     * @access public
     * @param integer id of group
     * @return mixed  boolean or DB Error object or false
     */
    function activateGroup($group_id)
    {
        if (!is_numeric($group_id)) {
            return false;
        }

        $query = 'UPDATE
                  ' . $this->prefix . 'groups
                SET
                  is_active = "Y"
                WHERE
                  group_id = ' . (int)$group_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Deactivate group
     *
     * @access public
     * @param  integer id of group
     * @return mixed   boolean or DB Error object
     */
    function deactivateGroup($group_id)
    {
        $query = 'UPDATE
                  ' . $this->prefix . 'groups
                SET
                  is_active = "N"
                WHERE
                  group_id = ' . (int)$group_id;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Imply right
     *
     * @access public
     * @param  integer id of right
     * @param  string  id of implied right
     * @return mixed   boolean or DB Error object
     */
    function implyRight($right_id, $implied_right_id)
    {
        // Imply Right
        $query = 'INSERT INTO
                  ' . $this->prefix . 'right_implied
                  (right_id, implied_right_id)
                VALUES
                  (
                    ' . (int) $right_id . ',
                    ' . (int) $implied_right_id . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Update implied status
        $this->_updateImpliedStatus($right_id);

        // Job done ...
        return true;
    }

    /**
     * Unimply right
     *
     * @access public
     * @param  integer id of right
     * @param  string  id of implied right
     * @return mixed   boolean or DB Error object
     */
    function unimplyRight($right_id, $implied_right_id)
    {
        // Unimply right
        $query = 'DELETE FROM
                  ' . $this->prefix . 'right_implied
                WHERE
                  right_id         = ' . (int)$right_id . ' AND
                  implied_right_id = ' . (int)$implied_right_id;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Update implied status
        $this->_updateImpliedStatus($right_id);

        // Job done ...
        return true;
    }

    /**
     * Update implied status of right
     *
     * @access private
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function _updateImpliedStatus($right_id)
    {
        // Are there any implied rights?
        $query = 'SELECT
                  Count(*)
                FROM
                  ' . $this->prefix . 'right_implied
                WHERE
                  right_id = ' . (int)$right_id;

        $count = $this->dbc->getOne($query);

        if (DB::isError($count)) {
            return $count;
        };

        $query = 'UPDATE
                  ' . $this->prefix . 'rights
                SET
                  has_implied = "' . (((int)$count > 0) ? 'Y' : 'N') . '"
                WHERE
                  right_id = ' . (int)$right_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    // XXX
    // function *Scope

    /**
     * Update right level of userRight
     *
     * @access public
     * @param  string  id of user
     * @param  integer id of right
     * @param  integer right level
     * @return mixed   boolean or DB Error object
     */
    function updateUserRight($authId, $right_id, $right_level)
    {
        $query = 'UPDATE
                  ' . $this->prefix . 'userrights
                SET
                  right_level = ' . (int)$right_level . '
                WHERE
                  user_id = ' . $this->_getPermUserId($authId) . ' AND
                  right_id = ' . (int)$right_id;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Grand right to group
     *
     * @access public
     * @param  integer id of group
     * @param  integer id of right
     * @param  integer right level
     * @return mixed   boolean or DB Error object
     */
    function grantGroupRight($group_id, $right_id, $right_level)
    {
        $query = 'INSERT INTO
                  ' . $this->prefix . 'grouprights
                  (group_id, right_id, right_level)
                VALUES
                  (
                    ' . (int)$group_id . ',
                    ' . (int)$right_id . ',
                    ' . (int)$right_level . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Revoke right from group
     *
     * @access public
     * @param  integer id of group
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function revokeGroupRight($group_id, $right_id)
    {
        $query = 'DELETE FROM
                  ' . $this->prefix . 'grouprights
                WHERE
                  group_id = ' . (int)$group_id . ' AND
                  right_id = ' . (int)$right_id;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Update right level of groupRight
     *
     * @access public
     * @param  integer id of group
     * @param  integer id of right
     * @param  integer right level
     * @return mixed   boolean or DB Error object
     */
    function updateGroupRight($group_id, $right_id, $right_level)
    {
        $query = 'UPDATE
                  ' . $this->prefix . 'grouprights
                SET
                  right_level = ' . (int)$right_level . '
                WHERE
                  group_id = ' . (int)$group_id . ' AND
                  right_id = ' . (int)$right_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Add User to Group
     *
     * @access public
     * @param  string  id of user
     * @param  integer id of group
     * @return mixed   boolean or DB Error object
     */
    function addUserToGroup($authId, $group_id)
    {
        $query = 'INSERT INTO
                  ' . $this->prefix . 'groupusers
                  (group_id, user_id)
                VALUES
                  (
                    ' . (int)$group_id . ',
                    ' . $this->_getPermUserId($authId) . '
                  )';
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Remove User from Group
     *
     * @access public
     * @param  string  id of user
     * @param  integer id of group
     * @return mixed   boolean or DB Error object
     */
    function removeUserFromGroup($authId, $group_id)
    {
        $query = 'DELETE FROM
                  ' . $this->prefix . 'groupusers
                WHERE
                  user_id  = ' . $this->_getPermUserId($authId) . ' AND
                  group_id = ' . (int) $group_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Get list of all groups
     *
     * This method accepts the following options...
     *  'where_user_id' = [PERM_USER_ID],
     *  'where_group_id' = [GROUP_ID],
     *  'where_owner_user_id' = [OWNER_USER_ID],
     *  'where_owner_group_id' = [OWNER_GROUP_ID],
     *  'where_is_active' = [BOOLEAN],
     *  'with_rights' = [BOOLEAN]
     *
     * @access public
     * @param  array  an array determining which fields and conditions to use
     * @param  mixed  do not use this parameter. Its for internal (recursive) use only
     * @return mixed array or DB Error object
     */
    function getGroups($options = null, $parentgroup = null)
    {
        static $groups;
        static $subgroups;
        $result = array();

        if ($parentgroup === null) {
            $query = 'SELECT
                      groups.group_id as group_id,
                      groups.owner_user_id as owner_user_id,
                      groups.owner_group_id as owner_group_id,
                      translations.name as name,
                      translations.comment as comment,
                      groups.is_active as is_active
                    FROM';

            if (isset($options['where_user_id'])
                    && is_numeric($options['where_user_id'])) {
                $query .= ' ' . $this->prefix . 'groupusers as groupusers,';
            }

            $query .= ' ' . $this->prefix . 'groups as groups,
                      ' . $this->prefix . 'translations as translations
                    WHERE';

            if (isset($options['where_user_id'])
                    && is_numeric($options['where_user_id'])) {
                $query .= ' groupusers.user_id = ' . (int)$options['where_user_id'] . ' AND
                          groupusers.group_id = groups.group_id AND';
            }

            if (isset($options['where_group_id'])
                     && is_numeric($options['where_group_id'])) {
                $query .= ' groups.group_id = ' . (int)$options['where_group_id'] . ' AND';
            }

            if (isset($options['where_owner_user_id'])
                    && is_numeric($options['where_owner_user_id'])) {
                $query .= ' groups.owner_user_id = ' . (int)$options['where_owner_user_id'] . ' AND';
            }

            if (isset($options['where_owner_group_id'])
                    && is_numeric($options['where_owner_group_id'])) {
                $query .= ' groups.owner_group_id = ' . (int)$options['where_owner_group_id'] . ' AND';
            }

            if (isset($options['where_is_active'])
                    && is_string($options['where_is_active'])) {
                $query .= ' groups.is_active = ' . $options['where_is_active'] . ' AND';
            }

            $query .= ' translations.section_id = groups.group_id AND
                      translations.section_type = '.LIVEUSER_SECTION_GROUP . ' AND
                      translations.language_id = ' . (int)$this->_langs[$this->getCurrentLanguage()];

            $_groups = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

            if (DB::isError($_groups)) {
                return $_groups;
            };

            if (!is_array($_groups)) {
                return $_groups;
            };

            $groups = array();
            foreach($_groups as $key => $value) {
                if ($options['with_rights'] == true) {
                    $_options = $options;
                    $_options['where_group_id'] = $value['group_id'];
                    $value['rights'] = $this->getRights($_options);
                };
                $groups[$value['group_id']] = $value;
            };

            $query = 'SELECT
                      subgroups.group_id as group_id,
                      subgroups.subgroup_id as subgroup_id
                    FROM
                      ' . $this->prefix . 'group_subgroups as subgroups';

            $subgroups = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

            if (DB::isError($subgroups)) {
                return $subgroups;
            };

            foreach($groups as $id => $group) {
                $in_array = false;
                foreach($subgroups as $subgroup) {
                    if ($id == $subgroup['subgroup_id']) {
                        $in_array = true;
                        break;
                    };
                };
                if (!$in_array) {
                    $result[$id] = $group;
                    $result[$id]['subgroups'] = $this->getGroups($options, $id);
                };
            };
        } else {
            foreach($subgroups as $subgroup) {
                if ($subgroup['group_id'] == $parentgroup) {
                    $result[$subgroup['subgroup_id']] = $groups[$subgroup['subgroup_id']];
                    $result[$subgroup['subgroup_id']]['subgroups'] = $this->getGroups($options, $subgroup['subgroup_id']);
                };
            };
        };

        return $result;
    }
}
?>