<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * LiveUser is a framework designed to be flexible.
 *
 * Since it is impossible to have a
 * "one size fits all" it takes a container
 * approach which should enable it to
 * be versatile enough to meet most needs.
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Include PEAR base class
 */
require_once 'PEAR.php';

/**#@+
 * Error related constants definition
 *
 * @var integer
 */
define('LIVEUSER_ERROR',                        -1);
define('LIVEUSER_NOT_SUPPORTED',                -2);
define('LIVEUSER_CONFIG_ERROR',                 -3);
define('LIVEUSER_MISSING_DEPS',                 -6);
define('LIVEUSER_MISSING_LOGINFUNCTION',        -7);
define('LIVEUSER_MISSING_LOGOUTFUNCTION',       -9);
define('LIVEUSER_COOKIE_ERROR',                -11);
define('LIVEUSER_AUTH_NOTFOUND',               -12);
/**#@-*/

/**#@+
 * Statuses of the current user.
 *
 * @var integer
 */
define('LIVEUSER_AUTH_IDLED',       -1);
define('LIVEUSER_AUTH_EXPIRED',     -2);
define('LIVEUSER_AUTH_WRONG_LOGIN', -3);
define('LIVEUSER_AUTH_ISINACTIVE',  -4);
/**#@-*/

/**
 * Class LiveUser - Login handling class
 *
 * Description:
 * This is a manager class for a user login system using the LiveUser
 * class. It creates a LiveUser object, takes care of the whole login
 * process and stores the LiveUser object in a session.
 *
 * You can also configure this class to try to connect to more than
 * one server that can store user information - each server requiring
 * a different backend class. This way you can for example create a login
 * system for a live website that first queries the local database and
 * if the requested user is not found, it tries to find im in your
 * company's LDAP server. That way you don't have to create lots of
 * user accounts for your employees so that they can access closed
 * sections of your website - everyone can use his existing account.
 *
 * NOTE: No browser output may be made before using this class, because
 * it will try to send HTTP headers such as cookies and redirects.
 *
 * Requirements:
 * - Should run on PHP version 4.1.0 or higher, tested only from 4.2.1 onwards
 *
 * Thanks to:
 * Bjoern Schotte, Kristian Koehntopp, Antonio Guerra
 *
 * @author   Markus Wolff       <wolff@21st.de>
 * @author   Bjoern Kraus       <krausbn@php.net>
 * @author   Lukas Smith        <smith@backendmedia.com>
 * @author   Pierre-Alain Joye  <pajoye@php.net>
 * @author   Arnaud Limbourg    <arnaud@php.net>
 * @version  $Id: LiveUser.php,v 1.32 2003/08/18 19:34:52 arnaud Exp $
 * @package  LiveUser
 */
class LiveUser
{
     /**
     * LiveUser options set in the configuration file
     *
     * This array contains private options defined by
     * the following associative keys:
     *
     * <code>
     *
     * array(
     *  'autoInit' => false/true,
     *  'session'  => array(
     *      'name'    => 'liveuser session name',
     *      'varname' => 'liveuser session var name'
     *  ),
     *  'login' => array(
     *      'method'   => 'get or post',
     *      'username' => 'Form input containing user handle',
     *      'password' => 'Form input containing password',
     *      'remember' => '(optional) Form checkbox containing <Remember Me> info',
     *      'function' => '(optional) Function to be called when accessing a page without logging in first',
     *      'force'    => 'Should the user be forced to login'
     *  ),
     *  'logout' => array(
     *      'trigger'  => 'GET or POST var that triggers the logout process',
     *      'redirect' => 'Page path to be redirected to after logout',
     *      'function' => '(optional) Function to be called when accessing a page without logging in first',
     *      'destroy'  => 'Whether to destroy the session on logout'
     *  ),
     * // The cookie options are optional. If they are specified, the Remember Me
     * // feature is activated.
     *  'cookie' => array(
     *      'name'     => 'Name of Remember Me cookie',
     *      'lifetime' => 'Cookie lifetime in days',
     *      'path'     => 'Cookie path',
     *      'domain'   => 'Cookie domain',
     *      'secret'   => 'Secret key used for cookie value encryption'
     *  ),
     *  'authContainers' => array(
     *      array('type' => 'DB',
     *            'connection'    => $db,
     *            'loginTimeout'  => 0,
     *            'expireTime'    => 3600,
     *            'idleTime'      => 1800,
     *            'allowDuplicateHandles' => 0,
     *            'authTable'     => 'liveuser_users',
     *            'authTableCols' => array('userId'    => 'user_id',
     *                                     'handle'    => 'handle',
     *                                     'password'  => 'password',
     *                                     'lastLogin' => 'lastlogin'
     *            )
     *      )
     *  ),
     *  'permContainer' => array(
     *      'type'       => 'DB_Complex',
     *      'connection' => $db,
     *      'prefix'     => 'liveuser_'
     *  )
     *
     * Other options in the configuration file relative to
     * the Auth and Perm containers depend on what the
     * containers expect. Refer to the Containers documentation.
     * The examples for containers provided are just general
     * do not reflect all the options for all containers.
     *
     * @access  private
     * @var     array
     */
    var $_options = array(
        'autoInit'=> false,
        'session' => array(
            'name'     => 'PHPSESSION',
            'varname'  => 'ludata'
        ),
        'login'   => array(
            'method'   => 'post',
            'username' => 'username',
            'password' => 'password',
            'force'    => false,
            'function' => '',
            'remember' => false
        ),
        'logout'  => array(
            'trigger'  => 'logout',
            'redirect' => '',
            'destroy'  => true,
            'function' => ''
        )
    );

    /**
     * The auth container object.
     *
     * @access private
     * @var    object
     */
    var $_auth = null;

    /**
     * The permission container object
     *
     * @access private
     * @var    object
     */
    var $_perm = null;

    /**
     * Nested array with the auth containers that shall be queried for user information.
     * Format:
     * <code>
     * array(1 => array("option1" => "value", ....))
     * </code>
     * Typical options are:
     * <ul>
     * - server: The adress of the server being queried (ie. "localhost").
     * - handle: The user name used to login for the server.
     * - password: The password used to login for the server.
     * - database: Name of the database containing user information (this is
     *   usually used only by RDBMS).
     * - baseDN: Obviously, this is what you need when using an LDAP server.
     * - connection: Present only if an existing connection shall be used. This
     *   contains a reference to an already existing connection resource or object.
     * - type: The container type. This option must always be present, otherwise
     *   the LoginManager can't include the correct container class definition.
     *</ul>
     *
     * @access private
     * @var    array
     */
    var $authContainers = array();

    /**
     * Name of the session variable that is used to store information on the
     * backend in which the auth data resides.
     * This is needed to include the
     * correct backend class definition on demand.
     *
     * @access private
     * @var    string
     */
    var $authContainerTypeVar = 'authContainerType';

    /**
     * Array of settings for the permission container to use for retrieving
     * user rights.
     * If set to false, no permission container will be used.
     * If that is the case, all calls to checkRight() will return false.
     * The array element 'type' must be present for the LoginManager to be able
     * to include the correct class definition (example: "DB_Complex").
     *
     * @acess private
     * @var   mixed
     */
    var $permContainer = false;

    /**
     * Has the auth session expired?
     *
     * @access private
     * @var    boolean
     * @see    init(), processLogout()
     */
    var $expired = false;

    /**
     * Is the maximum idletime over?
     *
     * @access private
     * @var    boolean
     * @see    init(), processLogout()
     */
    var $idled = false;

    /**
     * Current authentication status
     *
     * @access private
     * @var    string
     */
    var $status = null;

    /**
     * Whether an error was raised
     *
     * @access private
     * @var    boolean
     */
    var $_error = false;

    /**
     * Checks the given file and returns an object of the LoginManager.
     *
     * @access public
     * @param  mixed    $conf       The config file or the config array to configure.
     * @param  string   $handle     Handle of the user trying to authenticate
     * @param  string   $passwd     Password of the user trying to authenticate
     * @param  boolean  $logout     set to true if user wants to logout
     * @param  mixed    $confName   Name of array containing the configuration.
     * @return object   Returns an object.
     */
    function &factory($conf, $handle = '', $passwd = '', $logout = false, $confName = 'liveuserConfig')
    {
        $obj = &new LiveUser();

        if (!empty($conf)) {
            $init = $obj->_readConfig($conf, $confName);
            if (LiveUser::isError($init)) {
                return $init;
            }
        }

        if (!PEAR::isError($obj) &&
            isset($obj->_options['autoInit']) &&
            $obj->_options['autoInit']
        ) {
            $init = $obj->init($handle, $passwd, $logout);
            if (LiveUser::isError($init)) {
                return $init;
            }
        }

        return $obj;
    }

     /**
      * Class constructor.
      * Do not instantiate the class directly. Use the factory method
      *
      * @access protected
      * @see    factory()
      */
    function LiveUser()
    {
    }

    /**
     * Clobbers two arrays together
     * taken from the user notes of array_merge_recursive
     * used in LiveUser::_readConfig()
     *
     * @access private
     * @param  array    $a1   Array that should be clobbered
     * @param  array    $a2   Array that should be clobbered
     *
     * @return mixed array on success and false on error
     * @author kc@hireability.com
     */
    function _arrayMergeClobber($a1, $a2)
    {
        if (!is_array($a1) || !is_array($a2)) {
            return false;
        }
        $newarray = $a1;
        while (list($key, $val) = each($a2)) {
            if (is_array($val) && isset($newarray[$key]) && is_array($newarray[$key])) {
                $newarray[$key] = $this->_arrayMergeClobber($newarray[$key], $val);
            } else {
                $newarray[$key] = $val;
            }
        }
        return $newarray;
    }

    /**
     * Reads the configuration
     *
     * @access private
     * @param  mixed    $conf       Conf array or file path to configuration
     * @param  string   $confName   Name of array containing the configuration
     * @return mixed    true on success, PEAR_Error on failure
     */
    function _readConfig($conf, $confName)
    {
        if (is_array($conf)) {
            if (isset($conf['authContainers'])) {
                $this->authContainers = $conf['authContainers'];
                unset($conf['authContainers']);
            }
            if (isset($conf['permContainer'])) {
                $this->permContainer = $conf['permContainer'];
                unset($conf['permContainer']);
            }

            $this->_options = $this->_arrayMergeClobber($this->_options, $conf);

            return true;
        }

        if (!is_readable($conf)) {
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_CONFIG_ERROR, null, E_USER_WARNING,
                "Could not read the configuration file in LiveUser::readConfig()", 'LiveUser', true);
        }
        include_once($conf);
        if (isset(${$confName}) && is_array(${$confName})) {
            return $this->_readConfig(${$confName}, $confName);
        } else {
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_CONFIG_ERROR, null, E_USER_WARNING,
                "Configuration array not found in LiveUser::readConfig()", 'LiveUser', true);
        }
    }

    /**
     * Crypts data using mcrypt or userland if not available
     *
     * @access private
     * @param  boolean true to crypt, false to decrypt
     * @param  string  data to crypt
     * @return string  crypted data
     */
    function _cookieCryptMode($crypt, $data)
    {
        if (function_exists('mcrypt_module_open')) {
            $td = mcrypt_module_open('tripledes', '', 'ecb', '');
            $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size ($td), MCRYPT_RAND);
            mcrypt_generic_init($td, $this->_options['cookie']['secret'], $iv);
            if ($crypt) {
                $data = mcrypt_generic($td, $data);
            } else {
                $data = mdecrypt_generic($td, $data);
            }
            mcrypt_generic_deinit($td);
            mcrypt_module_close($td);
            unset($td, $iv);
        } else {
            include_once('Crypt/Rc4.php');
            $crypt =& new Crypt_RC4($this->_options['cookie']['secret']);
            if ($crypt) {
                $data = $crypt->crypt($data);
            } else {
                $data = $crypt->decrypt($data);
            }
            unset($crypt);
        }

        return $data;
    }

    /**
     * Sets an option.
     *
     * @access public
     * @param  string $option option name
     * @param  mixed  $value value for the option
     * @return mixed true or PEAR Error
     * @see    LiveUser::_options
     */
    function setOption($option, $value)
    {
        if (isset($this->options[$option])) {
            $this->options[$option] = $value;
            return true;
        }
        return PEAR::raiseError(null, LIVEUSER_CONFIG_ERROR, null, E_USER_WARNING,
            "unknown option $option", 'LiveUser', true);
    }

    /**
     * Returns the value of an option
     *
     * @access public
     * @param  string $option option name
     * @return mixed the option value
     */
    function getOption($option)
    {
        if (isset($this->options[$option])) {
            return $this->options[$option];
        }
        return PEAR::raiseError(null, LIVEUSER_CONFIG_ERROR, null, E_USER_WARNING,
            "unknown option $option", 'LiveUser', true);
    }

    /**
     * Get the Auth Container class instance of it exists
     *
     * @access public
     * @return mixed object or PEAR Error
     */
    function &getAuthContainer()
    {
        if (is_object($this->_auth)) {
            return $this->_auth;
        }
        return PEAR::raiseError(null, LIVEUSER_CONFIG_ERROR, null, E_USER_WARNING,
            "authentication container has not yet been instantianted", 'LiveUser', true);
    }

    /**
     * Get the Perm Container class instance of it exists
     *
     * @access public
     * @return mixed object or PEAR Error
     */
    function &getPermContainer()
    {
        if (is_object($this->_perm)) {
            return $this->_perm;
        }
        return PEAR::raiseError(null, LIVEUSER_CONFIG_ERROR, null, E_USER_WARNING,
            "permission container has not yet been instantianted", 'LiveUser', true);
    }

    /**
     * Tries to retrieve auth object from session.
     * If this fails, the class attempts a login based on cookie or form
     * information (depends on class settings).
     * Returns true if a auth object was successfully retrieved or created.
     * Otherwise, false is returned.
     *
     * @access public
     * @param  string   handle of the user trying to authenticate
     * @param  string   password of the user trying to authenticate
     * @param  boolean  set to true if user wants to logout
     * @return boolean  true if init process well, false otherwise
     */
    function init($handle = '', $passwd = '', $logout = false)
    {
        if ($this->_error) {
            return false;
        }

        // Set the name of the current session
        session_name($this->_options['session']['name']);
        // If there's no session yet, start it now
        session_start();

        // Determine if user wanted to log out
        $this->processLogout($logout);

        // Include needed permission container class
        if (is_array($this->permContainer)) {
            include_once 'LiveUser/Perm/Container/' . $this->permContainer['type'] . '.php';
        }

        // Try to fetch auth object from session
        if (!$this->retrieveAuthObject()) {
            if (is_null($this->_auth)) {
                $this->tryLogin($handle, $passwd);
            }
        }

        // Check if authentication session is expired.
        if (is_object($this->_auth) &&
            $this->_auth->loggedIn &&
            $this->_auth->expireTime > 0 &&
            $this->_auth->currentLogin > 0 &&
            ($this->_auth->currentLogin + $this->_auth->expireTime) < time()) {

            $this->expired = true;
            $this->status = LIVEUSER_AUTH_EXPIRED;
            $this->processLogout();
        }

        // Check if maximum idle time is reached.
        if (is_object($this->_auth) &&
            $this->_auth->loggedIn &&
            $this->_auth->idleTime > 0 &&
            isset($_SESSION[$this->_options['session']['varname']]['idle']) &&
            ($_SESSION[$this->_options['session']['varname']]['idle'] + $this->_auth->idleTime) < time())
        {
            $this->idled = true;
            $this->status = LIVEUSER_AUTH_IDLED;
            $this->processLogout();
        }

        $_SESSION[$this->_options['session']['varname']]['idle'] = time();

        // Force user login.
        if ($this->_options['login']['force'] && (!$this->isLoggedIn() || $this->status)) {
            if (!empty($this->_options['login']['function']) && function_exists($this->_options['login']['function'])) {
                call_user_func($this->_options['login']['function'], $this);
            }
        }

        // Return boolean that indicates whether a auth object has been created
        // or retrieved from session
        if (is_object($this->_auth)) {
            return true;
        } elseif (empty($this->_auth->handle)) {
            return false;
        } else {
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_AUTH_NOTFOUND, null, E_USER_WARNING,
                "Auth object not found in LiveUser::init()", 'LiveUser', true);
        }
    }

    /**
     * Tries to log the user in by trying all the Auth containers defined
     * in the configuration file until there is a success or failure.
     *
     * @access private
     * @param  string   handle of the user trying to authenticate
     * @param  string   password of the user trying to authenticate
     */
    function tryLogin($handle = '', $passwd = '')
    {
        if ($this->_error) {
            return false;
        }

        // handle and password not directly passed
        if (empty($handle)) {
            if (isset($this->_options['cookie']) && isset($_COOKIE[$this->_options['cookie']['name']])) {
                $cookieData = unserialize($_COOKIE[$this->_options['cookie']['name']]);
                if (count($cookieData) != 3) {
                    $this->_error = true;
                    return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null,E_USER_WARNING,
                        "Wrong data in cookie store in LiveUser::tryLogin()", 'LiveUser', true);
                }

                $serverData = $this->_readStoreCookiePasswdId($cookieData[0]);

                if ($serverData[0] != $cookieData[2]) {
                    $this->_error = true;
                    return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null, E_USER_WARNING,
                        "Passwords hashes do not match in cookie in LiveUser::tryLogin()", 'LiveUser', true);
                }

                $handle = $cookieData[1];
                $passwd = $serverData[1];
            } else {
                $httpvar = (strtolower($this->_options['login']['method']) == 'post') ? $_POST : $_GET;

                if (empty($httpvar[$this->_options['login']['username']]) ||
                    empty($httpvar[$this->_options['login']['password']]))
                {
                    if (!empty($this->_options['login']['function']) &&
                        function_exists($this->_options['login']['function']))
                    {
                        call_user_func($this->_options['login']['function'], $this);
                    }
                } else {
                    $handle = $httpvar[$this->_options['login']['username']];
                    $passwd = $httpvar[$this->_options['login']['password']];
                    if (get_magic_quotes_gpc()) {
                        $handle = stripslashes($handle);
                        $passwd = stripslashes($passwd);
                    }
                }
            }
        }

        if (empty($handle)) {
            return false;
        }

        $counter     = 0;
        $userFound   = false;
        $backend_cnt = count($this->authContainers);
        $backends    = &$this->authContainers;

        //loop into auth containers
        while ($userFound == false && $backend_cnt > $counter && $handle != '') {
            $classname = 'LiveUser_Auth_Container_'.$backends[$counter]['type'];
            include_once 'LiveUser/Auth/Container/' . $backends[$counter]['type'] . '.php';
            $auth = &new $classname($backends[$counter]);
            $auth->login($handle, $passwd, true);
            if ($auth->loggedIn) {
                $userFound = true;
                $this->_auth = $auth;
                $this->_auth->backendArrayIndex = $counter;

                // Create permission object
                if (is_array($this->permContainer)) {
                    $permClass = 'LiveUser_Perm_Container_' . $this->permContainer['type'];
                    include_once 'LiveUser/Perm/Container/' . $this->permContainer['type'] . '.php';
                    if (isset($this->permContainer['connection'])) {
                        $this->_perm = &new $permClass($this->permContainer['connection']);
                    } else {
                        $this->_perm = &new $permClass($this->permContainer);
                    }
                    $res = $this->_perm->init($this->_auth->authUserId);

                    if (DB::isError($res)) {
                        return $res;
                    }
                }

                $this->freeze($counter);
            } elseif ($auth->handle != '') {
                $this->status = LIVEUSER_AUTH_ISINACTIVE;
                $userFound  = true;
                $this->_auth = $auth;
                $this->_auth->backendArrayIndex = $counter;
                $this->freeze($counter);
            } else {
                $this->status = LIVEUSER_AUTH_WRONG_LOGIN;
            }
            $counter++;
        } // End loop into backends
    }

    /**
     * Gets auth and perm container objects back from session and tries
     * to give them an active database/whatever connection again
     *
     * @access private
     * @return mixed   false or void
     */
    function unfreeze()
    {
        if ($this->_error) {
            return false;
        }

        $this->_auth = unserialize($_SESSION[$this->_options['session']['varname']]['auth']);

        if (isset($_SESSION[$this->_options['session']['varname']]['perm'])) {
            $this->_perm = unserialize($_SESSION[$this->_options['session']['varname']]['perm']);
        }

        // Use the same backend connection as for the first login
        $i = $this->_auth->backendArrayIndex;

        //need deeper tests
        $this->_auth->unfreeze($this->authContainers[$i]);

        if (isset($this->permContainer['connection']) && is_object($this->_perm)) {
            $this->_perm->unfreeze($this->permContainer['connection']);
        } elseif (is_object($this->_perm)) {
            $this->_perm->unfreeze($this->permContainer);
        }
    }

    /**
     * Calls the container objects freeze methods which should close
     * their database connections. After that, the container objects are
     * being serialized and stored in the session.
     *
     * @access private
     * @param  integer
     */
    function freeze($counter)
    {
        if ($this->_error) {
            return false;
        }
        $this->_auth->freeze();
        if (is_object($this->_perm)) {
            $this->_perm->freeze();
        }

        // Bind objects to session
        $_SESSION[$this->_options['session']['varname']] = array();
        $_SESSION[$this->_options['session']['varname']]['auth'] = serialize($this->_auth);
        if (is_object($this->_perm)) {
            $_SESSION[$this->_options['session']['varname']]['perm'] = serialize($this->_perm);
        }
        $_SESSION[$this->authContainerTypeVar] = $this->authContainers[$counter]['type'];

        $this->setRememberCookie();
    }

    /**
     * If cookies are allowed, this method checks if the user wanted
     * a cookie to be set so he doesnt have to enter handle and password
     * for his next login. If true, it will set the cookie.
     *
     * @access private
     * @return boolean  true if the cookie can be set, false otherwise
     */
    function setRememberCookie()
    {
        if ($this->_error) {
            return false;
        }

        $httpvar = (strtolower($this->_options['login']['method']) == 'post') ? $_POST : $_GET;

        if (isset($this->_options['cookie']) && isset($httpvar[$this->_options['login']['remember']])) {
            // Calculate cookie timeout in days
            $cookieTimeout = time() + (86400 * $this->_options['cookie']['lifetime']);

            if (!empty($httpvar[$this->_options['login']['password']])) {
                $password = $httpvar[$this->_options['login']['password']];
                $handle   = $httpvar[$this->_options['login']['username']];
            } else {
                return false;
            }

            $store_id = md5($handle . $password);

            if (!$passwd_id = $this->_storeCookiePasswdId($password, $store_id)) {
                $this->_error = true;
                return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null, E_USER_WARNING,
                    "Cannot store cookie data in LiveUser::setRememberCookie()", 'LiveUser', true);
            }

            $setcookie = setcookie(
                          $this->_options['cookie']['name'],
                          serialize(array($store_id, $this->_auth->handle, $passwd_id)),
                          $cookieTimeout,
                          $this->_options['cookie']['path'],
                          $this->_options['cookie']['domain']);

            if ($setcookie) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * Determines the backend type of the auth object stored in the session,
     * includes the proper class definition and then imports the object from
     * the session.
     *
     * @access private
     * @return boolean
     */
    function retrieveAuthObject()
    {
        if ($this->_error) {
            return false;
        }

        $success = false;

        if (isset($_SESSION[$this->_options['session']['varname']]['auth'])) {
            $classFile  = 'LiveUser/Auth/Container/';
            $classFile .= $_SESSION[$this->authContainerTypeVar] . '.php';
            include_once $classFile;
            if (class_exists('LiveUser_Auth_Container_'.
                             $_SESSION[$this->authContainerTypeVar])) {
                $success = true;
                $this->unfreeze();
            }
        }
        return $success;
    }

    /**
     * If the logout parameter has been set via GET/POST or the user is
     * idled/expired, this destroys the session object.
     *
     * @access private
     * @param  boolean  set to true if user wants to logout
     * @return void
     * @see    init()
     */
    function processLogout($logout = false)
    {
        if ($this->_error) {
            return false;
        }

        // Has the user to be logged out?
        if ($logout ||
            isset($_GET[$this->_options['logout']['trigger']]) ||
            isset($_POST[$this->_options['logout']['trigger']]) ||
            $this->idled || $this->expired
        ) {
            $this->logout();
        }
    }

    /**
     * This destroys the session object.
     *
     * @access public
     * @return void
     */
    function logout()
    {
        if ($this->_error) {
            return false;
        }

        // If a callback function is set, call it
        if (!empty($this->_options['logout']['function']) && function_exists($this->_options['logout']['function'])) {
            call_user_func($this->_options['logout']['function'], $this);
        }

        // Yes indeed, destroy the session object now!
        unset($_SESSION[$this->_options['session']['varname']]);

        // If there's a cookie, kill that one too...
        if (isset($this->_options['cookie']) && isset($_COOKIE[$this->_options['cookie']['name']])) {
            $cookieKillTime = time() - 86400;
            setcookie($this->_options['cookie']['name'], '', $cookieKillTime,
                      $this->_options['cookie']['path'], $this->_options['cookie']['domain']);
        }

        // If the session should be destroyed, do so now...
        if ($this->_options['logout']['destroy'] == true) {
            session_destroy();
        }

        // Finally, set the LoginManager's internal auth object back to NULL
        $this->_auth = null;

        // If there's a URL to redirect to on logout, do so
        if (!empty($this->_options['logout']['redirect'])) {
            header('Location: ' . $this->_options['logout']['redirect']);
        }
    }

    /**
     * Wrapper method for the permission object's own checkRight method.
     *
     * @access public
     * @param  mixed    A right name or an array of rights.
     * @return boolean  true if the user has the right/rights false if not
     */
    function checkRight($rights)
    {
        if ($this->_error) {
            return false;
        }
        if (is_object($this->_perm)) {
            $hasright = false;

            if (is_array($rights)) {
                foreach($rights as $currentright) {
                    if ($this->_perm->checkRight($currentright)) {
                        $hasright = true;
                    } else {
                        $hasright = false;
                        break;
                    }
                }
            } else {
                // Remember: $rights is a single value at this point!
                $hasright = $this->_perm->checkRight($rights);
            }

            return $hasright;
        } else {
            return false;
        }
    }

    /**
     * Wrapper method for the permission object's own checkRightLevel method.
     *
     * @access public
     * @param  integer  right identifier
     * @param  integer  user identifier
     * @param  integer  group identifier
     * @return boolean
     */
    function checkRightLevel($right_id, $owner_user_id, $owner_group_id)
    {
        if ($this->_error) {
            return false;
        }
        if (is_object($this->_perm)) {
            return ($this->_perm->checkRightLevel($right_id, $owner_user_id, $owner_group_id));
        } else {
            return false;
        }
    }

    /**
     * Checks if a user is logged in.
     *
     * @access public
     * @return boolean
     */
    function isLoggedIn()
    {
        if ($this->_error) {
            return false;
        }
        if (!is_object($this->_auth)) {
            return false;
        } elseif ($this->_auth->loggedIn == true) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Function that determines if the user exists but hasn't yet been declared
     * "active" by an administrator. Use this to check if this was the reason
     * why a user was not able to login.
     * true == Not active
     * false == active
     *
     * @access public
     * @return boolean
     */
    function isInactive()
    {
        if ($this->_error) {
            return false;
        }
        if (is_object($this->_auth)
            && $this->_auth->isActive == 0
            && $this->_auth->handle != '') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * The user can set a function that will be called if the user
     * tries to access a page wihout logging in first. It will receive
     * the liveuser object. If an empty string or a non-existent function is passed
     * it deactivates the call.
     *
     * @access  public
     * @param   string  The name of the function to be called.
     * @return  mixed   void or PEAR_Error
     */
    function setLoginFunction($functionName)
    {
        if ($this->_error) {
            return false;
        }
        if (!empty($functionName) && function_exists($functionName)) {
            $this->_options['login']['function'] = $functionName;
        } else {
            return PEAR::raiseError(null, LIVEUSER_MISSING_LOGINFUNCTION, null, E_USER_WARNING,
                "Login function not found in LiveUser::setLoginFunction()", 'LiveUser', true);
        }
    }

    /**
     * The user can set a function that will be called if the user
     * wants to logout (by providing the appropriate GET-parameter).
     * If an empty string or a non-existent function is passed
     * it deactivates the call.
     * <b>Attention: Don't use a die() or exit() statement in your logout function.
     *            Otherwise the user can't be logged out properly.</b>
     *
     * @access  public
     * @param   string  The name of the function to be called.
     * @return  mixed   void or a PEAR_Error
     * @see     LiveUser::_options
     */
    function setLogoutFunction($functionName)
    {
        if ($this->_error) {
            return false;
        }
        if (!empty($functionName) && function_exists($functionName)) {
            $this->_options['logout']['function'] = $functionName;
        } else {
            return PEAR::raiseError(null, LIVEUSER_MISSING_LOGOUTFUNCTION, null, E_USER_WARNING,
                "Logout function not found in LiveUser::setLogoutFunction()", 'LiveUser', true);
        }
    }

    /**
     * Wrapper method for the auth object's own getHandle method.
     *
     * @access public
     * @return string   The handle of the current user.
     *                  Returns an empty string if no user logged.
     */
    function getHandle()
    {
        return !$this->_error && $this->_auth ?
                $this->_auth->getProperty('handle') : '';
    }

    /**
     * Wrapper method to access properties from the auth and
     * permission containers.
     *
     * @access public
     * @param  string   Name of the property to be returned.
     * @return mixed    NULL, a value or an array.
     */
    function getProperty($what)
    {
        if ($this->_error) {
            return false;
        }
        $that = null;
        if ($this->_auth && $this->_auth->getProperty($what) !== null) {
            $that = $this->_auth ? $this->_auth->getProperty($what) : null;
        } elseif ($this->_perm && $this->_perm->getProperty($what) !== null) {
            $that = $this->_perm ? $this->_perm->getProperty($what) : null;
        }
        return $that;
    }

    /**
     * Returns a one-dimensional array with all rights assigned
     * to this user. Array format depends on the optional parameter:
     * TRUE: array(intRight_ID => intRightLevel, ...)
     * FALSE array(intRight_ID, ...) [Default]
     * If no rights are available, FALSE is returned.
     *
     * @access public
     * @param  boolean  Return array with right_ids as key and level as value
     * @return mixed    an array of rights or false
     */
    function getRights($withLevels = false)
    {
      if (is_object($this->_perm)) {
          return($this->_perm->getRights($withLevels));
      }
      return false;
    }

    /**
     * Get the current status
     *
     * @access public
     * @return integer
     */
    function getStatus()
    {
        return $this->status;
    }

    /**
     * A "store" on the server contains the password and the
     * cookie id in an encrypted form.
     *
     * This method generates a md5 from given
     * password and writes it into the "store" along
     * with crypted password.
     *
     * Cookies are not secure but keeping
     * the password in plain text in the cookie
     * is not the best way to go. Since some
     * containers like LDAP need the clear text
     * password, we store an encrypted version
     * of the password using Crypt_Rc4 which provides
     * a simple two-way mechanism.
     *
     * To do this LiveUser needs access
     * to a writeable directory. If you do
     * no have access to the ini_get()
     * function please set a constant
     * named LIVEUSER_TMPDIR with an absolute
     * path to a writeable directory.
     *
     * @access private
     * @param  string   the password to store
     * @return boolean  true if success, false otherwise
     */
    function _storeCookiePasswdId($passwd, $store)
    {
        if (!defined('LIVEUSER_TMPDIR')) {
            define('LIVEUSER_TMPDIR', ini_get('session.save_path'));
        }

        if (!$fh = fopen(LIVEUSER_TMPDIR . "/$store.lu", 'w')) {
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null, E_USER_WARNING,
                "Cannot open file for writting in LiveUser::_storeCookiePasswdId()", 'LiveUser', true);
        }

        $data = serialize(array(md5($passwd), $passwd));

        $crypted_data = $this->_cookieCryptMode(true, $data);

        if (!fwrite($fh, $crypted_data)) {
            fclose($fh);
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null, E_USER_WARNING,
                "Cannot save cookie data in LiveUser::_storeCookiePasswdId()", 'LiveUser', true);
        }

        fclose($fh);

        return true;
    }

    /**
     * A "store" on the server contains the password and the
     * cookie id in an encrypted form.
     *
     * This method reads the data contained in it.
     *
     * @access private
     * @param  string the filename of the store
     * @return mixed  an array of the data, false otherwise
     */
    function _readStoreCookiePasswdId($store)
    {
        if (!defined('LIVEUSER_TMPDIR')) {
            define('LIVEUSER_TMPDIR', ini_get('session.save_path'));
        }

        if (!$fh = fopen(LIVEUSER_TMPDIR . "/$store.lu", 'r')) {
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null, E_USER_WARNING,
                "Cannot open file for reading in LiveUser::_readStoreCookiePasswdId()", 'LiveUser', true);
        }

        if (!$fields = fread($fh, 4096)) {
            fclose($fh);
            $this->_error = true;
            return PEAR::raiseError(null, LIVEUSER_COOKIE_ERROR, null, E_USER_WARNING,
                "Cannot read file in LiveUser::_readStoreCookiePasswdId()", 'LiveUser', true);
        }

        fclose($fh);

        $params = unserialize($this->_cookieCryptMode(false, $fields));

        return (count($params) == 2) ? $params : false;
    }
    /**
     * Tell whether a result from a LiveUser method is an error
     *
     * @access  public
     * @param   mixed   result code
     * @return  bool    whether value is an error
     */
    function isError($value)
    {
        return (is_object($value) && (get_class($value) == 'liveuser_error' || is_subclass_of($value, 'liveuser_error')));
    } // end func isError

    /**
     * Return a textual error message for a LiveUser error code
     *
     * @access  public
     * @param   int     error code
     * @return  string  error message
     */
    function errorMessage($value)
    {
        // make the variable static so that it only has to do the defining on the first call
        static $errorMessages;

        // define the varies error messages
        if (!isset($errorMessages)) {
            $errorMessages = array(
                LIVEUSER_ERROR                       => 'Unknown error',
                LIVEUSER_NOT_SUPPORTED               => 'Feature not supported',
                LIVEUSER_CONFIG_ERROR                => 'Config file error',
                LIVEUSER_MISSING_DEPS                => 'Missing package depedencies',
                LIVEUSER_MISSING_LOGINFUNCTION       => 'Login function not found',
                LIVEUSER_MISSING_LOGOUTFUNCTION      => 'Logout function not found',
                LIVEUSER_COOKIE_ERROR                => 'Remember Me cookie error',
                LIVEUSER_AUTH_NOTFOUND               => 'Cannot retrieve Auth object from session'
            );
        }

        // If this is an error object, then grab the corresponding error code
        if (LiveUser::isError($value)) {
            $value = $value->getCode();
        }

        // return the textual error message corresponding to the code
        return isset($errorMessages[$value]) ? $errorMessages[$value] : $errorMessages[LIVEUSER_ERROR];
    } // end func errorMessage
} // end class LiveUser

/**
 * LiveUser Error class
 *
 * @package  LiveUser
 */
class LiveUser_Error extends PEAR_Error
{
   /**
    * Prefix for all error messages
    *
    * @access private
    * @var    string
    */
    var $error_message_prefix = 'LiveUser Error: ';

   /**
    * Creates a liveuser error object, extending the PEAR_Error class
    *
    * @param  int   the error code
    * @param  int   the reaction to the error, either return, die or trigger/callback
    * @param  int   intensity of the error (PHP error code)
    * @param  mixed any information that can inform user as to nature of the error
    * @return PEAR_Error
    */
    function LiveUser_Error($code = LIVEUSER_ERROR, $mode = PEAR_ERROR_RETURN,
                         $level = E_USER_NOTICE, $debuginfo = null)
    {
        if (is_int($code)) {
            $this->PEAR_Error(LiveUser::errorMessage($code), $code, $mode, $level, $debuginfo);
        } else {
            $this->PEAR_Error("Invalid error code: $code", LIVEUSER_ERROR, $mode, $level, $debuginfo);
        }
    }
}
?>